package ru.myitschool.work.api;

import android.util.Log;

import androidx.annotation.NonNull;

import java.io.IOException;
import java.util.Objects;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import ru.myitschool.work.core.Constants;
import ru.myitschool.work.api.entity.Code;
import ru.myitschool.work.api.entity.Employee;

public class ControllerAPI<T extends HandlersResultAPI> {
    // Главный класс по работе с сервером. В качестве дженерик типа передаётся класс, в котором надо
    // имплементировать HandlersResultAPI. Потом нужно прописать в полученных методах работу
    // с результатами вызова API. И создав экземпляр этого класса вызвать нужный метод.
    //
    // Примерно так все должно выглядеть:
    //
    // public class LoginActivity ... implements HandlersResultAPI {
    //      private ControllerAPI controllerAPI;
    //
    //      @Override
    //      protected void onCreate(Bundle savedInstanceState) {
    //          ...
    //          controllerAPI = new ControllerAPI<LoginActivity>();
    //          ...
    //          controllerAPI.verificationLogin("Логин пользователя", LoginActivity.this);
    //      }
    //
    //      @Override
    //      public void handlerVerificationLogin(boolean result) {
    //          // Тут работа с результатами controllerAPI.verificationLogin
    //      }
    //
    //      @Override
    //      public void handlerGetEmployee(Employee employee) { }
    //
    //      @Override
    //      public void handlerVisit(boolean result) { }
    // }
    //
    // Всё это по факту один большой костыль, который придуман, чтобы обойти многопоточность.

    private final RequestsAPI managerAPI;

    public ControllerAPI() {
        Retrofit retrofit = new Retrofit.Builder()
                .baseUrl(Constants.SERVER_ADDRESS)
                .addConverterFactory(GsonConverterFactory.create())
                .build();
        managerAPI = retrofit.create(RequestsAPI.class);
    }

    public void verificationLogin(String login, T handler) {
        managerAPI.verificationLogin(login).enqueue(new Callback<>() {
            @Override
            public void onResponse(@NonNull Call<Void> call, @NonNull Response<Void> response) {
                Log.d("Test", "(Верификация логина) - Ответ от сервера: " + response.code());
                handler.handlerVerificationLogin(login, response.code() == 200);
            }

            @Override
            public void onFailure(@NonNull Call<Void> call, @NonNull Throwable t) {
                Log.e("Test", "(Верификация логина) - " + Objects.requireNonNull(t.getMessage()));
                handler.handlerVerificationLogin(login, false);
            }
        });
    }

    public void getEmployee(String login, T handler) {
        managerAPI.getEmployee(login).enqueue(new Callback<>() {
            @Override
            public void onResponse(@NonNull Call<Employee> call, @NonNull Response<Employee> response) {
                Log.d("Test", "(Получение пользователя) - Ответ от сервера: Тело - " + response.body() + " Код - " + response.code());
                handler.handlerGetEmployee(response.body());
            }

            @Override
            public void onFailure(@NonNull Call<Employee> call, @NonNull Throwable t) {
                Log.e("Test", "(Получение пользователя) - " + Objects.requireNonNull(t.getMessage()));
                handler.handlerGetEmployee(null);
            }
        });
    }

    public void visit(String login, Code code, T handler) throws IOException {
        managerAPI.visit(login, code).enqueue(new Callback<>() {
            @Override
            public void onResponse(Call<Void> call, Response<Void> response) {
                Log.d("Test", "(Проверка кода) - Ответ от сервера: Код - " + response.code());
                handler.handlerVisit(response.code());
            }

            @Override
            public void onFailure(Call<Void> call, Throwable t) {
                Log.e("Test", "(Получение пользователя) - " + Objects.requireNonNull(t.getMessage()));
                handler.handlerVisit(400);
            }
        });
    }
}