package ru.myitschool.work.ui.main

import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity.MODE_PRIVATE
import androidx.fragment.app.Fragment
import androidx.fragment.app.clearFragmentResultListener
import androidx.fragment.app.setFragmentResultListener
import androidx.navigation.fragment.findNavController
import ru.myitschool.work.R
import ru.myitschool.work.api.ControllerAPI
import ru.myitschool.work.api.HandlersResultAPI
import ru.myitschool.work.api.entity.Employee
import ru.myitschool.work.core.MyConstants
import ru.myitschool.work.databinding.FragmentMainBinding
import ru.myitschool.work.ui.login.LoginDestination
import ru.myitschool.work.ui.qr.scan.QrScanDestination
import ru.myitschool.work.ui.result.ResultDestination
import ru.myitschool.work.utils.DownloadImage
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

class MainFragment : Fragment(R.layout.fragment_main), HandlersResultAPI {
    private var _binding: FragmentMainBinding? = null
    private val binding: FragmentMainBinding get() = _binding!!

    private lateinit var controllerAPI: ControllerAPI<MainFragment>
    private lateinit var settings: SharedPreferences
    private lateinit var login: String

    private var flag = false

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentMainBinding.bind(view)
        controllerAPI = ControllerAPI<MainFragment>()
        settings = this.requireActivity().getSharedPreferences(MyConstants.PREFS_FILE, MODE_PRIVATE)

        if (flag && savedInstanceState == null) {
            Log.d("Test", "(1)")

            val editor = settings.edit()
            editor.putString("code", "Отмена")
            editor.putString("login", login)
            editor.apply()

            flag = false

            findNavController().navigate(ResultDestination)
            clearFragmentResultListener("qr_result")
        }

        init()
    }

    private fun init() {

        Log.d("Test", "(MainFragment) Проверяем зарегистрирован пользователь или нет.")
        if (settings.getString(MyConstants.PREF_LOGIN, MyConstants.DEF_VALUE) == MyConstants.DEF_VALUE) {
            Log.d("Test", "(MainFragment) Пользователь не зарегистрирован, запускаем активность регистрации.")
            findNavController().navigate(LoginDestination)
            return
        }

        login = settings.getString(MyConstants.PREF_LOGIN, MyConstants.DEF_VALUE).toString()
        Log.d("Test", "(MainFragment) Пользователь зарегистрирован, его login - $login")

        Log.d("Test", "(MainFragment) Отправляем запрос на получение данных пользователя.")
        controllerAPI.getEmployee(login, this)

        binding.logout.setOnClickListener {
            Log.d("Test", "(MainFragment) Запущен обработчик кнопки `Выход`.")

            settings.edit().clear().apply()
            Log.d("Test",
                "(MainFragment) Теперь логин пользователя: " + settings.getString(
                    MyConstants.PREF_LOGIN,
                    MyConstants.DEF_VALUE
                ).toString()
            )
            findNavController().navigate(LoginDestination)
        }

        binding.refresh.setOnClickListener {
            Log.d("Test", "(MainFragment) Запущен обработчик кнопки `Обновить`.")
            Log.d("Test", "(MainFragment) Отправляем запрос на получение данных пользователя.")
            controllerAPI.getEmployee(login, this)
        }

        binding.scan.setOnClickListener {
            setFragmentResultListener("qr_result") { _, bundle ->
                Log.d("Test", bundle.toString())
                val code: String = QrScanDestination.getDataIfExist(bundle) ?: return@setFragmentResultListener

                val editor = settings.edit()
                editor.putString("code", code)
                editor.putString("login", login)
                editor.apply()

                Log.d("Test", "(2)")

                flag = false
                findNavController().navigate(ResultDestination)
                clearFragmentResultListener("qr_result")
            }

            flag = true
            findNavController().navigate(QrScanDestination)
        }
    }

    override fun handlerVerificationLogin(login: String?, result: Boolean) {
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun handlerGetEmployee(employee: Employee?) {
        if (employee == null) {
            Log.e("Test", "(MainFragment) Сервер не вернул работника, вероятно, он не запущен.")

            allGone()
            binding.error.visibility = View.VISIBLE
            binding.refresh.visibility = View.VISIBLE

            return
        }

        Log.d("Test", "(MainFragment) Обработчик получения данных работника запустился!")
        Log.d("Test", "(MainFragment) employee: $employee")

        allVisibility()
        binding.error.visibility = View.GONE

        binding.fullname.text = employee.name
        binding.position.text = employee.position

        // Форматируем дату и время в вид yyyy-MM-dd HH:mm
        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
        val dateTime = LocalDateTime.parse(employee.lastVisit)
        val lastVisit = dateTime.format(formatter)

        binding.lastEntry.text = lastVisit
        downloadImage(employee.photo)

        Log.d(
            "Test", "(MainFragment) Записанные данные: fullname: ${binding.fullname.text}, " +
                    "position: ${binding.position.text}, lastEntry: ${binding.lastEntry.text}"
        )
    }

    override fun handlerVisit(result: Int) {
        // Тут ничего не должно быть.
    }

    private fun allGone() {
        binding.refresh.visibility = View.GONE
        binding.scan.visibility = View.GONE
        binding.logout.visibility = View.GONE
        binding.error.visibility = View.GONE
        binding.position.visibility = View.GONE
        binding.lastEntry.visibility = View.GONE
        binding.fullname.visibility = View.GONE
        binding.photo.visibility = View.GONE
    }

    private fun allVisibility() {
        binding.refresh.visibility = View.VISIBLE
        binding.scan.visibility = View.VISIBLE
        binding.logout.visibility = View.VISIBLE
        binding.error.visibility = View.VISIBLE
        binding.position.visibility = View.VISIBLE
        binding.lastEntry.visibility = View.VISIBLE
        binding.fullname.visibility = View.VISIBLE
        binding.photo.visibility = View.VISIBLE
    }

    private fun downloadImage(urlImage: String) {
        Log.d("Test", "(MainFragment) Начинаем скачивать изображение с URl: $urlImage")
        DownloadImage(binding.photo).execute(urlImage)
    }
}