package ru.myitschool.work.ui.result

import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.appcompat.app.AppCompatActivity.MODE_PRIVATE
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import ru.myitschool.work.R
import ru.myitschool.work.api.ControllerAPI
import ru.myitschool.work.api.HandlersResultAPI
import ru.myitschool.work.api.entity.Code
import ru.myitschool.work.api.entity.Employee
import ru.myitschool.work.core.MyConstants
import ru.myitschool.work.databinding.FragmentResultBinding
import ru.myitschool.work.ui.main.MainDestination
import java.io.IOException


class ResultFragment : Fragment(R.layout.fragment_result), HandlersResultAPI {
    private var _binding: FragmentResultBinding? = null
    private val binding: FragmentResultBinding get() = _binding!!

    private lateinit var controllerAPI: ControllerAPI<ResultFragment> // Работа с API
    private lateinit var settings: SharedPreferences

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentResultBinding.bind(view)

        controllerAPI = ControllerAPI<ResultFragment>()
        settings = this.requireActivity().getSharedPreferences(MyConstants.PREFS_FILE, MODE_PRIVATE)

        val code = settings.getString("code", "").toString()
        val login = settings.getString("login", "").toString()

        Log.d("Test", "(ResultLayoutActivity) code: $code, login: $login")

        if (code == "Отмена" || code == "") {
            binding.result.setText(R.string.cancelled)
        } else {
            try {
                controllerAPI.visit(login, Code(code.toLong()), this)
            } catch (e: NumberFormatException) {
                binding.result.setText(R.string.wrong)
            } catch (e: IOException) {
                binding.result.setText(R.string.wrong)
            }
        }

        binding.close.setOnClickListener {
            findNavController().navigate(MainDestination)
        }
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }

    override fun handlerVerificationLogin(login: String?, result: Boolean) {
    }

    override fun handlerGetEmployee(employee: Employee?) {
    }

    override fun handlerVisit(result: Int) {
        if (result == 200) {
            binding.result.setText(R.string.success)
        } else {
            binding.result.setText(R.string.wrong)
        }
    }
}